package Shape;

import Graph.readFile;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.CullFace;
import javafx.scene.shape.MeshView;
import javafx.scene.shape.TriangleMesh;
import javafx.scene.transform.Translate;

public class MovingObject {
    private readFile fileObject;
    private float[] texCoords;
    private float[] centroid;
    private TriangleMesh mesh;
    private MeshView triangleView;
    private Translate translate;
    private boolean dirtyTranslate;

    MovingObject() {
        texCoords = new float[2];
        mesh = new TriangleMesh();
        translate = new Translate();
        centroid = new float[3];
        dirtyTranslate = true;
    }

    public MovingObject(String[] parameters) {
        this();
        fileObject = new readFile(parameters);
        fileObject.makeVertexHash(fileObject.getPath());
        fileObject.makeMatrix();
        mesh.getPoints().setAll(fileObject.getVertices());
        mesh.getFaces().setAll(fileObject.getFaces());
        triangleView = new MeshView(mesh);
        setTempDefault();
        setCentroid();
    }

    public MovingObject(String filePath) {
        this();
        fileObject = new readFile(filePath);
        fileObject.makeMatrix();
        mesh.getPoints().setAll(fileObject.getVertices());
        mesh.getFaces().setAll(fileObject.getFaces());
        triangleView = new MeshView(mesh);
        setTempDefault();
        setCentroid();
    }

    public void setTexCoords(float t1, float t2) {
        texCoords[0] = t1;
        texCoords[1] = t2;
        mesh.getTexCoords().setAll(texCoords);
        triangleView.setMesh(mesh);
    }

    public void setTranslate(double x, double y, double z) {
        translate.setX(x);
        translate.setY(y);
        translate.setZ(z);
        triangleView.getTransforms().add(translate);
    }

    public void updateTranslate(double x, double y, double z) {
        translate.setX(x);
        translate.setY(y);
        translate.setZ(z);
    }

    public Translate getTranslate() {
        return translate;
    }

    public void setNewFilePath(String filePath) {
        fileObject = new readFile(filePath);
        fileObject.makeMatrix();
        mesh.getPoints().setAll(fileObject.getVertices());
        mesh.getFaces().setAll(fileObject.getFaces());
        setTexCoords(0,0);
        triangleView = new MeshView(mesh);
        setTempDefault();
        dirtyTranslate = true;
        setCentroid();
    }

    public void setTempDefault() {
        triangleView.setCullFace(CullFace.NONE);
        triangleView.setMaterial(new PhongMaterial(Color.BLUE));
        setTexCoords(0,0);
    }

    public MeshView getMesh() {
        return triangleView;
    }

    public void setCentroid() {
        if (dirtyTranslate) {
            float[] vertices = fileObject.getVertices();
            int count = 0;
            float x = 0;
            float y = 0;
            float z = 0;
            for(float ii : vertices) {
                int index = count % 3;
                if (index == 0) {
                    x += ii;
                } else if (index == 1) {
                    y += ii;
                } else {
                    z += ii;
                }
                count++;
            }
            centroid[0] = x/count;
            centroid[1] = y/count;
            centroid[2] = z/count;
        }
    }

    public float[] getCentroid() {
        return centroid;
    }

}
